# =====================================
# generator=datazen
# version=3.1.3
# hash=10ebf2374e8ea826d1191aa1df9ef925
# =====================================

"""
Useful defaults and other package metadata.
"""

DESCRIPTION = "A network-application TUI using textual."
PKG_NAME = "conntextual"
VERSION = "1.2.0"
